/*

	No More Studios

	May 6th, 2015

	Tic Tac Toe

	In this program, the user will try to beat the O's in 
	this game of tic tac toe. Try to get three in a row and
	win!

	Challenge:

	Try to modify the code to allow the player
	to pick between choosing O's or X's.
*/
#include <iostream>
#include <string>
#include <time.h>

#include <cmath>
using namespace std;

// Functions
void displayMainMenu();
void initGame();
void drawGame();
void updateBoard(char selection);
void initComputerTurn();
bool validInput(char selection);
bool checkWin(char player);
void newBoard();

// Variables
char gameBoard[3][3];
int totalTurns = 0;

int main() {

	newBoard();
	displayMainMenu();
	initGame();
	return 0;
}

void displayMainMenu() { // simply the function that displays the main menu
	system("cls");
	cout << "|||||| |||| |||| // |||||| |||||| ||||  //  |||||| ||||| ||||\n";
	cout << "  ||    ||  ||   //   ||   ||--|| ||    //    ||   || || ||--\n";
	cout << "  ||   |||| |||| //   ||   ||  || ||||  //    ||   ||||| ||||\n\n";

	cout << "Welcome to Tic-Tac-Toe.\n";
	cout << "The goal of the game is to get three in a row.\n\n";
	cout << "Press enter to start the game...";

	cin.ignore(10000, '\n'); // to remove anything that the user might enter on the main menu
}

void initGame() {
	
	bool gameOver = false;
	char selection;
	while(!gameOver) { //game loop
		drawGame();
		cout << "\n\n";
		
		cout << "Type the number where you wish to place your X, then hit enter. Turn #: " << totalTurns << "\n";
		cin >> selection;

		while(!validInput(selection)){
			cin.ignore(10000, '\n');
			cout << "\nInvalid selection, please input a different number: ";
			cin >> selection;
		}
		updateBoard(selection);
		totalTurns += 1;
		if(checkWin('X')) {
			drawGame();
			cout << "\n\nYOU WIN!!!!!\n";
			cout << "Press enter to restart game...";
			newBoard();
			cin.get();
		} else if(totalTurns == 5) {
			drawGame();
			cout << "\n\nDRAW!\n";
			cout << "Press enter to restart game...";
			newBoard();
			cin.get();
		} else {
			initComputerTurn();
			if(checkWin('O')) {
				drawGame();
				cout << "\n\nSORRY, THE O's WON.\n";
				cout << "Press enter to restart game...";
				newBoard();
				cin.get();
			}
		}
		cin.ignore(10000, '\n');
	}
}

void updateBoard(char selection) { //function finds the number in the array that the user inputted and changes it to the X
	for(int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			if(gameBoard[i][j] == selection) {
				gameBoard[i][j] = 'X';
			}
		}
	}
}

void newBoard() {
	// Restarts a new game board in case user wants to play the game multiple times.
	totalTurns = 0;
	int counter = 1;
	for(int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			gameBoard[i][j] = counter+48; // +48 because putting in the number counter as a char makes it into ASCII characters.
			counter += 1;
		}
	}
}

bool validInput(char selection) {
	// This function makes sure the user enters appropriate input. If not, then
	// the program rejects it (returns false) and requires the user to enter another input.
	if(selection >= 48 && selection <= 57) {
		for(int i = 0; i < 3; i++) {
			for(int j = 0; j < 3; j++) {
				if(gameBoard[i][j] == selection) {
					return true;
				}
			}
		}
		return false;
	} else {
		return false;
	}
}

void drawGame() { 
	// Function just draws the game board onto the console window and
	// also puts the lines on the grid.
	system("cls");
	for(int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			cout << gameBoard[i][j];
			if(((j + 1) % 3) != 0) {
				cout << "|";
			}
		}
		if(i < 2) {
			cout << "\n-----\n";
		}
	}
}

bool checkWin(char player) {
	// This function is for checking if any game-winning moves are made by
	// either the player, or the computer.
	if(gameBoard[0][0] == player && gameBoard[0][1] == player && gameBoard[0][2] == player) {
		return true;
	} else if(gameBoard[1][0] == player && gameBoard[1][1] == player && gameBoard[1][2] == player) {
		return true;
	} else if(gameBoard[2][0] == player && gameBoard[2][1] == player && gameBoard[2][2] == player) {
		return true;
	} else if(gameBoard[0][0] == player && gameBoard[1][0] == player && gameBoard[2][0] == player) {
		return true;
	} else if(gameBoard[0][1] == player && gameBoard[1][1] == player && gameBoard[2][1] == player) {
		return true;
	} else if(gameBoard[0][2] == player && gameBoard[1][2] == player && gameBoard[2][2] == player) {
		return true;
	} else if(gameBoard[0][0] == player && gameBoard[1][1] == player && gameBoard[2][2] == player) {
		return true;
	} else if(gameBoard[0][2] == player && gameBoard[1][1] == player && gameBoard[2][0] == player) {
		return true;
	} else {
		return false;
	}
}

void initComputerTurn() {
	bool successfulTurn = false;
	double randomNumber = 0;
	// Computer turn is randomly generated so it might not be a challenge to beat it.
	// However, sometimes it can be surprising!
	srand(time(NULL));
	randomNumber = rand() % 9 + 1;

	randomNumber -= 0.1;			   // These two lines are just to complete the
	randomNumber = ceil(randomNumber); // math library requirement; however, they can
									   // be removed without affecting the program.

	while(!successfulTurn) {
		for(int i = 0; i < 3; i++) {
			for(int j = 0; j < 3; j++) {
				if(gameBoard[i][j] == randomNumber+48) {
					gameBoard[i][j] = 'O';
					successfulTurn = true;
				}
			}
		}
		randomNumber = rand() % 9 + 1;
	}
}